#!/usr/bin/env python
# -*- coding: utf-8 -*-

import sys
import unittest
from inspect import getfile
from os.path import dirname

import django

from rainbowtests import colors


class ColorfulOut:
    """Wrapper around sys.sdout to make it's output colorful"""

    def write(self, arg, **kwargs):
        return sys.stdout.write(colors.blue(arg), **kwargs)


class RainbowTextTestResult(unittest.runner.TextTestResult):
    """The color-enhanced Text output for a test run."""

    def addSuccess(self, test):
        unittest.TestResult.addSuccess(self, test)
        if self.showAll:
            self.stream.writeln(colors.green("ok"))
        elif self.dots:
            self.stream.write(colors.green("."))
            self.stream.flush()

    def addError(self, test, err):
        unittest.TestResult.addError(self, test, err)
        if self.showAll:
            self.stream.writeln(colors.red("ERROR"))
        elif self.dots:
            self.stream.write(colors.red('E'))
            self.stream.flush()

    def addFailure(self, test, err):
        unittest.TestResult.addFailure(self, test, err)
        if self.showAll:
            self.stream.writeln(colors.red("FAIL"))
        elif self.dots:
            self.stream.write(colors.red('F'))
            self.stream.flush()

    def printErrorList(self, flavour, errors):
        """Print colorful test output:

        * Error and Failure headers are in Red.
        * The First/Last lines (Traceback heading and Exception) are Cyan.
        # Non-Django lines are Yellow.

        """

        # Find the path where django is installed.
        django_path = dirname(getfile(django))

        for test, err in errors:
            self.stream.writeln(colors.red(self.separator1))
            out = "{0}: {1}".format(flavour, self.getDescription(test))
            self.stream.writeln(colors.red(out))
            self.stream.writeln(colors.red(self.separator2))
            tb = "{0}".format(err)
            lines = tb.strip().split("\n")

            # Temporarily remove the first & last lines of the Traceback.
            if len(lines) > 2:
                first_line = colors.cyan(lines.pop(0))  # Traceback header
                last_line = colors.cyan(lines.pop(-1))  # The Exception
            else:
                first_line, last_line = ('', '')

            # The traceback is organized into pairs; a path and a line of code,
            # and the pair looks something like this:
            #
            # >>> File "/path/to/module.py", line 123, in method_name
            # >>>     some_line_of_code
            #
            # So, we'll group every two lines of the traceback, and highlight
            # the pertinent parts

            i = 0  # remember which line of the Traceback we're highlighting.
            groups = zip(*[lines[x::2] for x in range(2)])
            for path, line_of_code in groups:
                if django_path not in path:
                    lines[i] = colors.yellow(path)
                    lines[i + 1] = colors.yellow(line_of_code)
                i += 2

            lines.insert(0, first_line)
            lines.append(last_line)
            self.stream.writeln("\n".join(lines))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# NOTE: DjangoTestSuiteRunner was removed in Django 1.8
# We should remove it at some point?

import unittest

from django.test.simple import DjangoTestSuiteRunner

from rainbowtests import messages
from rainbowtests.test.core import RainbowTextTestResult


class RainbowTestSuiteRunner(DjangoTestSuiteRunner):
    """Replacement Test Suite Runner for Django."""
    def run_suite(self, suite, **kwargs):

        runner = unittest.TextTestRunner(
            verbosity=self.verbosity, failfast=self.failfast)
        runner.resultclass = RainbowTextTestResult
        result = runner.run(suite)

        if result.wasSuccessful():
            runner.stream.writeln(messages.random_happy())
        else:
            runner.stream.writeln(messages.random_sad())
        return result



from django.utils import termcolors

blue = termcolors.make_style(opts=('bold'), fg='blue')
cyan = termcolors.make_style(opts=('bold'), fg='cyan')
green = termcolors.make_style(fg='green')
magenta = termcolors.make_style(opts=('bold'), fg='magenta')
red = termcolors.make_style(opts=('bold'), fg='red')
white = termcolors.make_style(opts=('bold',), fg='white')
yellow = termcolors.make_style(opts=('bold'), fg='yellow')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import unittest

from django.conf import settings
from django.test.runner import DiscoverRunner

from rainbowtests import messages
from rainbowtests.test.core import ColorfulOut, RainbowTextTestResult

try:
    import coverage
except ImportError:
    coverage = None


class RainbowDiscoverRunner(DiscoverRunner):
    """Replacement for Django's DiscoverRunner"""

    def run_suite(self, suite, **kwargs):
        if not hasattr(self, "test_runner"):
            # This was added as a clas attribute in Django 1.7
            self.test_runner = unittest.TextTestRunner

        runner = self.test_runner(
            verbosity=self.verbosity,
            failfast=self.failfast,
        )
        runner.resultclass = RainbowTextTestResult
        result = runner.run(suite)

        runner.stream.writeln(messages.random(result.wasSuccessful()))
        return result


class RainbowDiscoverCoverageRunner(RainbowDiscoverRunner):
    """Replacement for Django's DiscoverRunner with coverage support"""

    def __init__(self, *args, **kwargs):
        super(RainbowDiscoverCoverageRunner, self).__init__(*args, **kwargs)
        self._cov = None
        if coverage:
            self._cov = coverage.coverage()
            self._cov.start()

    def suite_result(self, suite, result, **kwargs):
        result = super(RainbowDiscoverCoverageRunner, self).suite_result(
            suite, result, **kwargs
        )
        if coverage and self._cov:
            self._cov.stop()
            self._cov.report(file=ColorfulOut())
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from random import choice

from rainbowtests import colors

happy_messages = []
sad_messages = []

happy_messages.append("""

        (づ｡◕‿‿◕｡)づ・。*。✧・゜・。
        ✧。*・゜゜・✧。・゜゜SUCCESS!!・。
        *。・゜*✧✿。*。✧・゜゜・✧。✧。*・

""")

happy_messages.append("""

        (☞ﾟ∀ﾟ)☞ You rock!
        All tests passed.

""")

happy_messages.append("""

        ヽ(´▽`)/ Hallelujah!
        Praise FSM. Success.

""")

happy_messages.append("""

        (｡♥‿♥｡) ・。♥。✧・゜・✿
        ✧。*・Your code loves you.✧♥
        ✿。・ All tests passed!✿。*
        ✧・゜♥・✧。♥。*・。♥。✧・♥・✿
""")

sad_messages.append("""

        （╯°□°）╯︵ ┻━┻
        AAAAAAAAAARGGGGGGGGGG!!!

""")

sad_messages.append("""

        ლ(ಠ益ಠლ)
        Bloody tests failed AGAIN!

""")


def random_happy():
    return colors.green(choice(happy_messages))


def random_sad():
    return colors.red(choice(sad_messages))


def random(success):
    return random_happy() if success else random_sad()
